@@ -43,7 +43,6 @@ import ansible.errors
 import ansible.plugins.connection
 
 import mitogen.core
-import mitogen.utils
 
 import ansible_mitogen.mixins
 import ansible_mitogen.parsing
@@ -51,6 +50,7 @@ import ansible_mitogen.process
 import ansible_mitogen.services
 import ansible_mitogen.target
 import ansible_mitogen.transport_config
+import ansible_mitogen.utils.unsafe
 
 
 LOG = logging.getLogger(__name__)
@@ -484,6 +484,7 @@ class Connection(ansible.plugins.connection.ConnectionBase):
     login_context = None
 
     #: Only sudo, su, and doas are supported for now.
+    # Ansible ConnectionBase attribute, removed in Ansible >= 2.8
     become_methods = ['sudo', 'su', 'doas']
 
     #: Dict containing init_child() return value as recorded at startup by
@@ -521,15 +522,6 @@ class Connection(ansible.plugins.connection.ConnectionBase):
     # set by `_get_task_vars()` for interpreter discovery
     _action = None
 
-    def __del__(self):
-        """
-        Ansible cannot be trusted to always call close() e.g. the synchronize
-        action constructs a local connection like this. So provide a destructor
-        in the hopes of catching these cases.
-        """
-        # https://github.com/dw/mitogen/issues/140
-        self.close()
-
     def on_action_run(self, task_vars, delegate_to_hostname, loader_basedir):
         """
         Invoked by ActionModuleMixin to indicate a new task is about to start
@@ -684,6 +676,9 @@ class Connection(ansible.plugins.connection.ConnectionBase):
 
     @property
     def connected(self):
+        """
+        Ansible connection plugin property. Used by ansible-connection command.
+        """
         return self.context is not None
 
     def _spec_from_via(self, proxied_inventory_name, via_spec):
@@ -802,7 +797,7 @@ class Connection(ansible.plugins.connection.ConnectionBase):
                 call_context=self.binding.get_service_context(),
                 service_name='ansible_mitogen.services.ContextService',
                 method_name='get',
-                stack=mitogen.utils.cast(list(stack)),
+                stack=ansible_mitogen.utils.unsafe.cast(list(stack)),
             )
         except mitogen.core.CallError:
             LOG.warning('Connection failed; stack configuration was:\n%s',
@@ -842,14 +837,18 @@ class Connection(ansible.plugins.connection.ConnectionBase):
         the _connect_*() service calls defined above to cause the master
         process to establish the real connection on our behalf, or return a
         reference to the existing one.
+
+        Ansible connection plugin method.
         """
+        # In some Ansible connection plugins this method returns self.
+        # However nothing I've found uses it, it's not even assigned.
         if self.connected:
             return
 
         inventory_name, stack = self._build_stack()
         worker_model = ansible_mitogen.process.get_worker_model()
         self.binding = worker_model.get_binding(
-            mitogen.utils.cast(inventory_name)
+            ansible_mitogen.utils.unsafe.cast(inventory_name)
         )
         self._connect_stack(stack)
 
@@ -880,6 +879,8 @@ class Connection(ansible.plugins.connection.ConnectionBase):
         Arrange for the mitogen.master.Router running in the worker to
         gracefully shut down, and wait for shutdown to complete. Safe to call
         multiple times.
+
+        Ansible connection plugin method.
         """
         self._put_connection()
         if self.binding:
@@ -896,6 +897,8 @@ class Connection(ansible.plugins.connection.ConnectionBase):
         any local state we hold for the connection, returns the Connection to
         the 'disconnected' state, and informs ContextService the connection is
         bad somehow, and should be shut down and discarded.
+
+        Ansible connection plugin method.
         """
         if self._play_context.remote_addr is None:
             # <2.5.6 incorrectly populate PlayContext for reset_connection
@@ -930,7 +933,7 @@ class Connection(ansible.plugins.connection.ConnectionBase):
                 call_context=binding.get_service_context(),
                 service_name='ansible_mitogen.services.ContextService',
                 method_name='reset',
-                stack=mitogen.utils.cast(list(stack)),
+                stack=ansible_mitogen.utils.unsafe.cast(list(stack)),
             )
         finally:
             binding.close()
@@ -1002,12 +1005,14 @@ class Connection(ansible.plugins.connection.ConnectionBase):
             Data to supply on ``stdin`` of the process.
         :returns:
             (return code, stdout bytes, stderr bytes)
+
+        Ansible connection plugin method.
         """
         emulate_tty = (not in_data and sudoable)
         rc, stdout, stderr = self.get_chain().call(
             ansible_mitogen.target.exec_command,
-            cmd=mitogen.utils.cast(cmd),
-            in_data=mitogen.utils.cast(in_data),
+            cmd=ansible_mitogen.utils.unsafe.cast(cmd),
+            in_data=ansible_mitogen.utils.unsafe.cast(in_data),
             chdir=mitogen_chdir or self.get_default_cwd(),
             emulate_tty=emulate_tty,
         )
@@ -1027,12 +1032,14 @@ class Connection(ansible.plugins.connection.ConnectionBase):
             Remote filesystem path to read.
         :param str out_path:
             Local filesystem path to write.
+
+        Ansible connection plugin method.
         """
         self._connect()
         ansible_mitogen.target.transfer_file(
             context=self.context,
             # in_path may be AnsibleUnicode
-            in_path=mitogen.utils.cast(in_path),
+            in_path=ansible_mitogen.utils.unsafe.cast(in_path),
             out_path=out_path
         )
 
@@ -1050,7 +1057,7 @@ class Connection(ansible.plugins.connection.ConnectionBase):
         """
         self.get_chain().call_no_reply(
             ansible_mitogen.target.write_path,
-            mitogen.utils.cast(out_path),
+            ansible_mitogen.utils.unsafe.cast(out_path),
             mitogen.core.Blob(data),
             mode=mode,
             utimes=utimes,
@@ -1076,6 +1083,8 @@ class Connection(ansible.plugins.connection.ConnectionBase):
             Local filesystem path to read.
         :param str out_path:
             Remote filesystem path to write.
+
+        Ansible connection plugin method.
         """
         try:
             st = os.stat(in_path)
@@ -1110,7 +1119,7 @@ class Connection(ansible.plugins.connection.ConnectionBase):
             call_context=self.binding.get_service_context(),
             service_name='mitogen.service.FileService',
             method_name='register',
-            path=mitogen.utils.cast(in_path)
+            path=ansible_mitogen.utils.unsafe.cast(in_path)
         )
 
         # For now this must remain synchronous, as the action plug-in may have
